import * as cdk from 'aws-cdk-lib';
import * as constructs from 'constructs';
/**
 * CloudFormation equivalent of random_string
 *
 * @schema CfnStringProps
 */
export interface CfnStringProps {
    /**
     * @schema CfnStringProps#Keepers
     */
    readonly keepers?: KeepersDefinition[];
    /**
     * @schema CfnStringProps#Length
     */
    readonly length: number;
    /**
     * @schema CfnStringProps#Lower
     */
    readonly lower?: boolean;
    /**
     * @schema CfnStringProps#MinLower
     */
    readonly minLower?: number;
    /**
     * @schema CfnStringProps#MinNumeric
     */
    readonly minNumeric?: number;
    /**
     * @schema CfnStringProps#MinSpecial
     */
    readonly minSpecial?: number;
    /**
     * @schema CfnStringProps#MinUpper
     */
    readonly minUpper?: number;
    /**
     * @schema CfnStringProps#Number
     */
    readonly number?: boolean;
    /**
     * @schema CfnStringProps#OverrideSpecial
     */
    readonly overrideSpecial?: string;
    /**
     * @schema CfnStringProps#Special
     */
    readonly special?: boolean;
    /**
     * @schema CfnStringProps#Upper
     */
    readonly upper?: boolean;
}
/**
 * Converts an object of type 'CfnStringProps' to JSON representation.
 */
export declare function toJson_CfnStringProps(obj: CfnStringProps | undefined): Record<string, any> | undefined;
/**
 * @schema KeepersDefinition
 */
export interface KeepersDefinition {
    /**
     * @schema KeepersDefinition#MapKey
     */
    readonly mapKey: string;
    /**
     * @schema KeepersDefinition#MapValue
     */
    readonly mapValue: string;
}
/**
 * Converts an object of type 'KeepersDefinition' to JSON representation.
 */
export declare function toJson_KeepersDefinition(obj: KeepersDefinition | undefined): Record<string, any> | undefined;
/**
 * A CloudFormation `TF::Random::String`
 *
 * @cloudformationResource TF::Random::String
 * @stability external
 * @link https://github.com/iann0036/cfn-tf-custom-types.git
 */
export declare class CfnString extends cdk.CfnResource {
    /**
    * The CloudFormation resource type name for this resource class.
    */
    static readonly CFN_RESOURCE_TYPE_NAME = "TF::Random::String";
    /**
     * Resource props.
     */
    readonly props: CfnStringProps;
    /**
     * Attribute `TF::Random::String.tfcfnid`
     * @link https://github.com/iann0036/cfn-tf-custom-types.git
     */
    readonly attrTfcfnid: string;
    /**
     * Attribute `TF::Random::String.Id`
     * @link https://github.com/iann0036/cfn-tf-custom-types.git
     */
    readonly attrId: string;
    /**
     * Attribute `TF::Random::String.Result`
     * @link https://github.com/iann0036/cfn-tf-custom-types.git
     */
    readonly attrResult: string;
    /**
     * Create a new `TF::Random::String`.
     *
     * @param scope - scope in which this resource is defined
     * @param id    - scoped id of the resource
     * @param props - resource properties
     */
    constructor(scope: constructs.Construct, id: string, props: CfnStringProps);
}
